@@ -4,6 +4,7 @@ from pathlib import Path
 from fontTools.ttLib import TTFont
 from beziers.path import BezierPath
 from beziers.path.geometricshapes import Rectangle
+from beziers.utils.linesweep import bbox_intersections
 from beziers.point import Point
 from beziers.boundingbox import BoundingBox
 from glyphtools import categorize_glyph
@@ -13,8 +14,6 @@ from typing import NamedTuple
 class Collision(NamedTuple):
     glyph1: str
     glyph2: str
-    ix1: int
-    ix2: int
     path1: BezierPath
     path2: BezierPath
     point: Point
@@ -51,6 +50,7 @@ class Collidoscope:
         """
         self.fontfilename = fontfilename
         self.glyphcache = {}
+        self.direction = direction
         if ttFont:
             self.font = ttFont
             self.fontbinary = ttFont.reader.file.read()
@@ -152,20 +152,16 @@ class Collidoscope:
         if not (g1["glyphbounds"].overlaps(g2["glyphbounds"])): return []
         # print("Glyph bounds overlap")
 
-        overlappingPathBounds = []
-        for ix1,p1 in enumerate(g1["pathbounds"]):
-            for ix2,p2 in enumerate(g2["pathbounds"]):
-                if p1.overlaps(p2):
-                    overlappingPathBounds.append( (ix1,ix2) )
+        overlappingPathBounds = bbox_intersections(g1["paths"], g2["paths"])
 
         if not overlappingPathBounds: return []
 
         overlappingPaths = {}
-        for ix1, ix2 in overlappingPathBounds:
-            p1 = g1["paths"][ix1]
-            p2 = g2["paths"][ix2]
-            for s1 in p1.asSegments():
-              for s2 in p2.asSegments():
+        for p1, p2 in overlappingPathBounds:
+            left_segs = p1.asSegments()
+            right_segs = p2.asSegments()
+            overlappingSegBounds = bbox_intersections(left_segs, right_segs)
+            for s1,s2 in overlappingSegBounds:
                 intersects = s1.intersections(s2)
                 if len(intersects)>0:
                     overlappingPaths[(p1,p2)] = Collision(
@@ -173,18 +169,21 @@ class Collidoscope:
                         glyph2=g2["name"],
                         path1=p1,
                         path2=p2,
-                        ix1=ix1,
-                        ix2=ix2,
                         point=intersects[0].point
                         )
         return list(overlappingPaths.values())
 
-    def get_glyphs(self, text):
+    def get_glyphs(self, text, buf=None):
         """Returns an list of dictionaries representing a shaped string.
 
+        Args:
+            text: text to check
+            buf: (Optional) already shaped uharfbuzz buffer.
+
         This is the first step in collision detection; the dictionaries
         returned can be fed to ``draw_overlaps`` and ``has_collisions``."""
-        buf = self.shape_a_text(text)
+        if not buf:
+            buf = self.shape_a_text(text)
         glyf = self.font["glyf"]
         cursor = 0
         glyphs = []
@@ -249,7 +248,7 @@ class Collidoscope:
         glyphs = glyphs_in
         if self.direction == "rtl":
             glyphs = list(reversed(glyphs))
-        if self.rules["faraway"]:
+        if "faraway" in self.rules:
             for firstIx, first in enumerate(glyphs):
                 passedBases = 0
                 nonAdjacent = firstIx + 1
@@ -288,7 +287,7 @@ class Collidoscope:
             for firstIx in range(0,len(glyphs)-1):
                 first = glyphs[firstIx]
                 second = glyphs[firstIx+1]
-                if self.rules["cursive"]:
+                if "cursive" in self.rules and self.rules["cursive"]:
                     firstHasAnchors = any([x.hasAnchor for x in first["paths"]])
                     secondHasAnchors = any([x.hasAnchor for x in first["paths"]])
                     if firstHasAnchors or secondHasAnchors:
@@ -296,7 +295,7 @@ class Collidoscope:
                         overlaps = list(filter(lambda x: ((x.path1.hasAnchor and not x.path2.hasAnchor) or (x.path2.hasAnchor and not x.path1.hasAnchor)), overlaps))
                         if not overlaps: continue
                         return overlaps
-                if self.rules["area"] > 0:
+                if "area" in self.rules:
                     overlaps = self.find_overlaps(first, second)
                     if not overlaps: continue
                     newoverlaps = []
@@ -306,7 +305,7 @@ class Collidoscope:
                             ia = i.area
                             # print("Intersection area: %i Path 1 area: %i Path 2 area: %i" % (ia, p1.area, p2.area))
                             if ia > i1.path1.area * self.rules["area"] or ia > i1.path2.area*self.rules["area"]:
-                                newoverlaps.append(i)
+                                newoverlaps.append(i1)
                     if newoverlaps:
                         return newoverlaps
         return False
